//+------------------------------------------------------------------+
//|                                              Disparity Index.mq4 |
//|                                                         Linuxser |
//|                                          contact me in Forex TSD |
//|                                                                  |
//| A technical indicator that measures the relative position of the |
//| most recent closing price to a selected moving average and       |
//| reports the value as a percentage.                               |
//| A value greater than zero suggests that the asset is gaining     |
//| upward momentum, while a value less than zero can be interpreted |
//|  as a sign that selling pressure is increasing.                  |
//| Extreme values of this indicator can be a very useful tool for   |
//| contrarian investors to foretell periods of exhaustion. Once the |
//| price is excessively pushed in one direction, there are very few |
//| investors to take the other side of the transaction when the     |
//| participants wish to close their position, ultimately leading    |
//| to a price reversal. Similar to the ROC indicator, important     |
//| signals are generated when the indicator crosses over the zero   |
//| line because it is an early signal that momentum is building.    |
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  Silver
#property indicator_color2  DeepSkyBlue
#property indicator_color3  PaleVioletRed
#property indicator_width1  2
#property indicator_level1  0

//
//
//
//
//

extern string TimeFrame               = "Current time frame";
extern int    DispPeriod              = 13;
extern bool   drawDivergences         = false;
extern bool   drawIndicatorTrendLines = true;
extern bool   drawPriceTrendLines     = true;
extern color  colorBullishDivergence  = DeepSkyBlue;
extern color  colorBearishDivergence  = PaleVioletRed;
extern string UniqueID                = "DispIndex1";
extern bool   Interpolate             = false;

extern bool   alertsOn                = false;
extern bool   alertsOnCurrent         = true;
extern bool   alertsMessage           = true;
extern bool   alertsSound             = false;
extern bool   alertsEmail             = false;

//
//
//
//
//

double DispBuffer[];
double trend[];
double bullishDivergence[];
double bearishDivergence[];
string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;
string shortName;

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,DispBuffer);
   SetIndexBuffer(1,bullishDivergence); SetIndexStyle(1, DRAW_ARROW); SetIndexArrow(1, 233);
   SetIndexBuffer(2,bearishDivergence); SetIndexStyle(2, DRAW_ARROW); SetIndexArrow(2, 234);
   SetIndexBuffer(3,trend);
   
   //
   //
   //
   //
   //
         
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame=="calculateValue");
      if (calculateValue)
      {
         int s = StringFind(UniqueID,":",0);
            shortName = UniqueID;
            UniqueID  = StringSubstr(UniqueID,0,s);
            return(0);
      }            
      timeFrame = stringToTimeFrame(TimeFrame);
      shortName = UniqueID+": "+timeFrameToString(timeFrame)+" Disparity Index("+DispPeriod+")";

   IndicatorShortName(shortName);
   return(0);
}

//
//
//
//
//

int deinit()
{
   int length = StringLen(UniqueID);
   for(int i=ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i);  if(StringSubstr(name,0,length) == UniqueID)  ObjectDelete(name);   
   }
   return(0);
}




//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
//
//

int start()
{
   int limit,i,counted_bars = IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { DispBuffer[0] = limit; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      for(i=limit; i>=0; i--)
      {
         double ma = iMA(NULL,0,DispPeriod,0,MODE_EMA,PRICE_CLOSE,i);
            
            DispBuffer[i]        = 100*(iClose(NULL,timeFrame,i)-ma)/ma;
            bullishDivergence[i] = EMPTY_VALUE;
            bearishDivergence[i] = EMPTY_VALUE;
            trend[i]      = trend[i+1];
               if (DispBuffer[i]>0) trend[i] =  1;
               if (DispBuffer[i]<0) trend[i] = -1;
               if (drawDivergences)
               {
                  CatchBullishDivergence(i);
                  CatchBearishDivergence(i);
               }           
      }
      manageAlerts();
      return(0);
   }                           

   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
      
      DispBuffer[i]        = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",DispPeriod,drawDivergences,drawIndicatorTrendLines,drawPriceTrendLines,colorBullishDivergence,colorBearishDivergence,shortName,0,y);
      bullishDivergence[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",DispPeriod,drawDivergences,drawIndicatorTrendLines,drawPriceTrendLines,colorBullishDivergence,colorBearishDivergence,shortName,1,y);
      bearishDivergence[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",DispPeriod,drawDivergences,drawIndicatorTrendLines,drawPriceTrendLines,colorBullishDivergence,colorBearishDivergence,shortName,2,y);
      trend[i]             = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",DispPeriod,drawDivergences,drawIndicatorTrendLines,drawPriceTrendLines,colorBullishDivergence,colorBearishDivergence,shortName,3,y);
            if (bullishDivergence[i]!=EMPTY_VALUE && bullishDivergence[i+1]!=EMPTY_VALUE) bullishDivergence[i+1]=EMPTY_VALUE;
            if (bearishDivergence[i]!=EMPTY_VALUE && bearishDivergence[i+1]!=EMPTY_VALUE) bearishDivergence[i+1]=EMPTY_VALUE;

            //
            //
            //
            //
            //
            
            if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
            datetime time = iTime(NULL,timeFrame,y);
               for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(int k = 1; k < n; k++)
                  DispBuffer[i+k] = DispBuffer[i] + (DispBuffer[i+n] - DispBuffer[i])*k/n;
   }
   manageAlerts();
   return(0);
}


//+------------------------------------------------------------------
//|                                                                 
//+------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar]== 1) doAlert(whichBar,"crossed level 0 up");
         if (trend[whichBar]==-1) doAlert(whichBar,"crossed level 0 down");
      }         
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Disparity index ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Disparity index "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define arrowsDisplacement 0.0001

void CatchBullishDivergence(int shift)
{
   shift++;
         bullishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(UniqueID+"l"+DoubleToStr(Time[shift],0));
            ObjectDelete(UniqueID+"l"+"os" + DoubleToStr(Time[shift],0));            
   if(!IsIndicatorLow(shift)) return;  

   //
   //
   //
   //
   //
      
   int currentLow = shift;
   int lastLow = GetIndicatorLastLow(shift+1);

   //
   //
   //
   //
   //
    
   if(DispBuffer[currentLow] > DispBuffer[lastLow] && Low[currentLow] < Low[lastLow])
   {
      bullishDivergence[currentLow] = DispBuffer[currentLow] - arrowsDisplacement;
      if(drawPriceTrendLines == true)
                 DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],colorBullishDivergence,STYLE_SOLID);
      if(drawIndicatorTrendLines == true)
                 DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],DispBuffer[currentLow],DispBuffer[lastLow],colorBullishDivergence,STYLE_SOLID);
   }
     
   //
   //
   //
   //
   //
        
   if(DispBuffer[currentLow] < DispBuffer[lastLow] && Low[currentLow] > Low[lastLow])
   {
      bullishDivergence[currentLow] = DispBuffer[currentLow] - arrowsDisplacement;
      if(drawPriceTrendLines == true)
                 DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow], colorBullishDivergence, STYLE_DOT);
      if(drawIndicatorTrendLines == true)                            
                 DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],DispBuffer[currentLow],DispBuffer[lastLow], colorBullishDivergence, STYLE_DOT);
   }
}

//
//
//
//
//

void CatchBearishDivergence(int shift)
{
   shift++; 
         bearishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(UniqueID+"h"+DoubleToStr(Time[shift],0));
            ObjectDelete(UniqueID+"h"+"os" + DoubleToStr(Time[shift],0));            
   if(IsIndicatorPeak(shift) == false) return;
   int currentPeak = shift;
   int lastPeak = GetIndicatorLastPeak(shift+1);

   //
   //
   //
   //
   //
      
   if(DispBuffer[currentPeak] < DispBuffer[lastPeak] && High[currentPeak]>High[lastPeak])
   {
       bearishDivergence[currentPeak] = DispBuffer[currentPeak] + arrowsDisplacement;
       if(drawPriceTrendLines == true)
               DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],colorBearishDivergence,STYLE_SOLID);
       if(drawIndicatorTrendLines == true)
               DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],DispBuffer[currentPeak],DispBuffer[lastPeak],colorBearishDivergence,STYLE_SOLID);
   }
   
   //
   //
   //
   //
   //

   if(DispBuffer[currentPeak] > DispBuffer[lastPeak] && High[currentPeak] < High[lastPeak])
   {
      bearishDivergence[currentPeak] = DispBuffer[currentPeak] + arrowsDisplacement;
      if (drawPriceTrendLines == true)
               DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak], colorBearishDivergence, STYLE_DOT);
      if (drawIndicatorTrendLines == true)
               DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],DispBuffer[currentPeak],DispBuffer[lastPeak], colorBearishDivergence, STYLE_DOT);
   }   
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

bool IsIndicatorPeak(int shift)
{
   if(DispBuffer[shift] >= DispBuffer[shift+1] && DispBuffer[shift] > DispBuffer[shift+2] && DispBuffer[shift] > DispBuffer[shift-1])
         return(true);
   else  return(false);
}
bool IsIndicatorLow(int shift)
{
   if(DispBuffer[shift] <= DispBuffer[shift+1] && DispBuffer[shift] < DispBuffer[shift+2] && DispBuffer[shift] < DispBuffer[shift-1])
         return(true);
   else  return(false);
}
int GetIndicatorLastPeak(int shift)
{
   for(int i = shift+5; i < Bars; i++)
         if (DispBuffer[i] >= DispBuffer[i+1] && DispBuffer[i] > DispBuffer[i+2] &&  DispBuffer[i] >= DispBuffer[i-1] && DispBuffer[i] > DispBuffer[i-2])   return(i);
   return(-1);
}
int GetIndicatorLastLow(int shift)
{
   for(int i = shift+5; i < Bars; i++)
         if(DispBuffer[i] <= DispBuffer[i+1] && DispBuffer[i] < DispBuffer[i+2] && DispBuffer[i] <= DispBuffer[i-1] && DispBuffer[i] < DispBuffer[i-2]) return(i);
    return(-1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void DrawPriceTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
   string label = UniqueID+first+"os"+DoubleToStr(t1,0);
   ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, 0, t1+Period()*60-1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}
void DrawIndicatorTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
   int indicatorWindow = WindowFind(shortName);
   if (indicatorWindow < 0) return;
   
   string label = UniqueID+first+DoubleToStr(t1,0);
   ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, indicatorWindow, t1+Period()*60-1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}